package cc.toph.tutorialmod.block;

import java.util.function.Supplier;

import cc.toph.tutorialmod.TutorialMod;
import cc.toph.tutorialmod.item.ModItems;
import net.minecraft.util.valueproviders.UniformInt;
import net.minecraft.world.item.BlockItem;
import net.minecraft.world.item.Item;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.level.block.DropExperienceBlock;
import net.minecraft.world.level.block.SoundType;
import net.minecraft.world.level.block.state.BlockBehaviour;
import net.minecraftforge.eventbus.api.IEventBus;
import net.minecraftforge.registries.DeferredRegister;
import net.minecraftforge.registries.ForgeRegistries;
import net.minecraftforge.registries.RegistryObject;

public class ModBlocks {

	// Deferrer //
	public static final DeferredRegister<Block> BLOCKS = DeferredRegister.create(ForgeRegistries.BLOCKS,
			TutorialMod.MODID);

	// Blocks //
	// @formatter:off
	public static final RegistryObject<Block> SAPPHIRE_BLOCK = 
		registerBlock("sapphire_block",
			() -> new Block(
				BlockBehaviour.Properties.copy(Blocks.IRON_BLOCK)
					.sound(SoundType.AMETHYST)));

	public static final RegistryObject<Block> RAW_SAPPHIRE_BLOCK = 
		registerBlock("raw_sapphire_block",
			() -> new Block(
				BlockBehaviour.Properties.copy(Blocks.RAW_IRON_BLOCK)
					.sound(SoundType.AMETHYST)));

	public static final RegistryObject<Block> SAPPHIRE_ORE = 
		registerBlock("sapphire_ore",
			() -> new DropExperienceBlock(
				BlockBehaviour.Properties.copy(Blocks.STONE)
					.strength(2f)
						.sound(SoundType.AMETHYST_CLUSTER)
							.requiresCorrectToolForDrops(), UniformInt.of(3, 6)));
	
	public static final RegistryObject<Block> DEEPSLATE_SAPPHIRE_ORE = 
		registerBlock("deepslate_sapphire_ore",
			() -> new DropExperienceBlock(
				BlockBehaviour.Properties.copy(Blocks.DEEPSLATE_DIAMOND_ORE)
					.strength(3f)
						.sound(SoundType.DEEPSLATE)
							.requiresCorrectToolForDrops(), UniformInt.of(4, 6)));
	
	public static final RegistryObject<Block> NETHER_SAPPHIRE_ORE = 
		registerBlock("nether_sapphire_ore",
			() -> new DropExperienceBlock(
				BlockBehaviour.Properties.copy(Blocks.NETHER_GOLD_ORE)
					.strength(1f)
						.sound(SoundType.NETHER_ORE)
							.requiresCorrectToolForDrops(), UniformInt.of(2, 5)));

	public static final RegistryObject<Block> END_STONE_SAPPHIRE_ORE = 
		registerBlock("end_stone_sapphire_ore",
			() -> new DropExperienceBlock(
				BlockBehaviour.Properties.copy(Blocks.END_STONE)
					.strength(2f)
						.sound(SoundType.NETHER_ORE)
							.requiresCorrectToolForDrops(), UniformInt.of(10, 30)));
	
	// @formatter:on
	// Methods //
	public static void register(IEventBus eventBus) {
		BLOCKS.register(eventBus);
	}

	private static <T extends Block> RegistryObject<T> registerBlock(String name, Supplier<T> block) {

		RegistryObject<T> toReturn = BLOCKS.register(name, block);
		registerBlockItem(name, toReturn);

		return toReturn;
	}

	private static <T extends Block> RegistryObject<Item> registerBlockItem(String name, RegistryObject<T> block) {
		return ModItems.ITEMS.register(name, () -> new BlockItem(block.get(), new Item.Properties()));
	}

}
